@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: EphyPromptService.cpp 7426 2007-09-13 13:18:57Z chpe $
+ *  $Id: EphyPromptService.cpp 8286 2008-06-21 11:24:43Z jmouette $
  */
 
 #include "mozilla-config.h"
@@ -36,10 +36,21 @@
 #include "ephy-gui.h"
 #include "ephy-debug.h"
 
-#include "AutoJSContextStack.h"
-#include "AutoWindowModalState.h"
+#include "AutoModalDialog.h"
 #include "EphyUtils.h"
 
+#ifdef HAVE_GECKO_1_9
+#include <nsIStringBundle.h>
+#include "nsIChannel.h"
+#include "nsIProxiedChannel.h"
+#include "nsIProxyInfo.h"
+#include "nsNetCID.h"
+#include "nsIURI.h"
+#include "nsNetUtil.h"
+#include "nsIIDNService.h"
+#include "nsIAuthInformation.h"
+#endif /* HAVE_GECKO_1_9 */
+
 #include "EphyPromptService.h"
 
 #define TIMEOUT			1000 /* ms */
@@ -166,7 +177,7 @@ Prompter::Prompter (const char *aStock,
 		text = ConvertAndTruncateString (aText, MAX_MESSAGE_LENGTH);
 	}
 
-	label = gtk_label_new (text);
+	label = gtk_label_new (text ? text : "");
 	g_free (text);
 
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
@@ -322,7 +333,7 @@ Prompter::AddCheckbox (const PRUnichar *aText,
 	if (!aState || !aText) return;
 
 	char *label = ConvertAndEscapeButtonText (aText, 2 * MAX_BUTTON_TEXT_LENGTH);
-	mCheck = gtk_check_button_new_with_mnemonic (label);
+	mCheck = gtk_check_button_new_with_mnemonic (label ? label : "");
 	g_free (label);
 
 	gtk_label_set_line_wrap (GTK_LABEL (GTK_BIN (mCheck)->child), TRUE);
@@ -470,12 +481,9 @@ Prompter::Run (PRBool *aSuccess)
 	}
 #endif
 
-	nsresult rv;
-	AutoJSContextStack stack;
-	rv = stack.Init ();
-	if (NS_FAILED (rv)) return rv;
-
-	AutoWindowModalState modalState (mWindow);
+        AutoModalDialog modalDialog (mWindow, PR_TRUE);
+        if (!modalDialog.ShouldShow ())
+                return GTK_RESPONSE_CANCEL;
 
 	if (mDelay)
 	{
@@ -491,7 +499,7 @@ Prompter::Run (PRBool *aSuccess)
 
 	GtkWidget *widget = GTK_WIDGET (mDialog);
 	gtk_widget_show (widget);
-	mResponse = gtk_dialog_run (mDialog);
+	mResponse = modalDialog.Run (mDialog);
 	gtk_widget_hide (widget);
 
 	g_object_set_data (G_OBJECT (mDialog), TIMEOUT_DATA_KEY, NULL);
@@ -601,7 +609,10 @@ Prompter::ConvertAndTruncateString (const PRUnichar *aText,
 	char *converted = g_utf16_to_utf8 ((gunichar2*) aText, aMaxLength,
 					    &n_read, &n_written, NULL);
 	/* FIXME loop from the end while !g_unichar_isspace (char)? */
+        if (!converted)
+                return NULL;
 
+        g_strdelimit (converted, "\001\002\003\004\005\006\007\010\011\012\013\014\015\016\017\020\021\022\023\024\025\026\027\030\031\032\033\034\035\036\037", ' ');
 	return converted;
 }
 
@@ -642,8 +653,12 @@ Prompter::ConvertAndEscapeButtonText(const PRUnichar *aText,
 	return escaped;
 }
 
-/* FIXME: needs THREADSAFE? */
-#if HAVE_NSINONBLOCKINGALERTSERVICE_H
+#ifdef HAVE_GECKO_1_9
+NS_IMPL_THREADSAFE_ISUPPORTS3 (EphyPromptService,
+                               nsIPromptService,
+                               nsIPromptService2,
+                               nsINonBlockingAlertService)
+#elif defined(HAVE_NSINONBLOCKINGALERTSERVICE_H)
 NS_IMPL_ISUPPORTS2 (EphyPromptService,
 		    nsIPromptService,
 		    nsINonBlockingAlertService)
@@ -886,3 +901,223 @@ EphyPromptService::ShowNonBlockingAlert (nsIDOMWindow *aParent,
 }
 
 #endif /* HAVE_NSINONBLOCKINGALERTSERVICE_H */
+
+#ifdef HAVE_GECKO_1_9
+
+static void
+NS_GetAuthHostPort(nsIChannel* aChannel, nsIAuthInformation* aAuthInfo,
+                   PRBool machineProcessing, nsCString& host, PRInt32* port)
+{
+  nsCOMPtr<nsIURI> uri;
+  nsresult rv = aChannel->GetURI(getter_AddRefs(uri));
+  if (NS_FAILED(rv))
+    return;
+
+  // Have to distinguish proxy auth and host auth here...
+  PRUint32 flags;
+  aAuthInfo->GetFlags(&flags);
+  if (flags & nsIAuthInformation::AUTH_PROXY) {
+    nsCOMPtr<nsIProxiedChannel> proxied(do_QueryInterface(aChannel));
+    NS_ASSERTION(proxied, "proxy auth needs nsIProxiedChannel");
+
+    nsCOMPtr<nsIProxyInfo> info;
+    proxied->GetProxyInfo(getter_AddRefs(info));
+    NS_ASSERTION(info, "proxy auth needs nsIProxyInfo");
+
+    nsCAutoString idnhost;
+    info->GetHost(idnhost);
+    info->GetPort(port);
+
+    if (machineProcessing) {
+      nsCOMPtr<nsIIDNService> idnService =
+        do_GetService(NS_IDNSERVICE_CONTRACTID);
+      if (idnService) {
+        idnService->ConvertUTF8toACE(idnhost, host);
+      } else {
+        // Not much we can do here...
+        host = idnhost;
+      }
+    } else {
+      host = idnhost;
+    }
+  } else {
+    if (machineProcessing) {
+      uri->GetAsciiHost(host);
+      *port = NS_GetRealPort(uri);
+    } else {
+      uri->GetHost(host);
+      uri->GetPort(port);
+    }
+  }
+}
+
+static nsresult
+MakeDialogText(nsIChannel* aChannel, nsIAuthInformation* aAuthInfo,
+               nsString& message)
+{
+  nsresult rv;
+  nsCOMPtr<nsIStringBundleService> bundleSvc =
+    do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<nsIStringBundle> bundle;
+  rv = bundleSvc->CreateBundle("chrome://global/locale/prompts.properties",
+                               getter_AddRefs(bundle));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  // figure out what message to display...
+  nsCAutoString host;
+  PRInt32 port;
+  NS_GetAuthHostPort(aChannel, aAuthInfo, PR_FALSE, host, &port);
+
+  nsAutoString displayHost;
+  CopyUTF8toUTF16(host, displayHost);
+
+  nsCOMPtr<nsIURI> uri;
+  aChannel->GetURI(getter_AddRefs(uri));
+
+  nsCAutoString scheme;
+  uri->GetScheme(scheme);
+
+  nsAutoString username;
+  aAuthInfo->GetUsername(username);
+
+  PRUint32 flags;
+  aAuthInfo->GetFlags(&flags);
+  PRBool proxyAuth = (flags & nsIAuthInformation::AUTH_PROXY) != 0;
+
+  nsAutoString realm;
+  aAuthInfo->GetRealm(realm);
+
+  // Append the port if it was specified
+  if (port != -1) {
+    displayHost.Append(PRUnichar(':'));
+    displayHost.AppendInt(port);
+  }
+
+#ifdef HAVE_GECKO_1_9
+  NS_NAMED_LITERAL_STRING(proxyText, "EnterLoginForProxy");
+  NS_NAMED_LITERAL_STRING(originText, "EnterLoginForRealm");
+#else
+  NS_NAMED_LITERAL_STRING(proxyText, "EnterUserPasswordForProxy");
+  NS_NAMED_LITERAL_STRING(originText, "EnterUserPasswordForRealm");
+#endif /* HAVE_GECKO_1_9 */
+  NS_NAMED_LITERAL_STRING(noRealmText, "EnterUserPasswordFor");
+  NS_NAMED_LITERAL_STRING(passwordText, "EnterPasswordFor");
+
+  const PRUnichar *text;
+  if (proxyAuth) {
+    text = proxyText.get();
+  } else {
+    text = originText.get();
+
+    // prepend "scheme://"
+    nsAutoString schemeU; 
+    CopyASCIItoUTF16(scheme, schemeU);
+    schemeU.AppendLiteral("://");
+    displayHost.Insert(schemeU, 0);
+  }
+
+  const PRUnichar *strings[] = { realm.get(), displayHost.get() };
+  PRUint32 count = NS_ARRAY_LENGTH(strings);
+
+  if (flags & nsIAuthInformation::ONLY_PASSWORD) {
+    text = passwordText.get();
+    strings[0] = username.get();
+  } else if (!proxyAuth && realm.IsEmpty()) {
+    text = noRealmText.get();
+    count--;
+    strings[0] = strings[1];
+  }
+
+  rv = bundle->FormatStringFromName(text, strings, count, getter_Copies(message));
+  return rv;
+}
+
+/* static */ nsresult
+EphyPromptService::PromptPasswordAdapter(nsIPromptService* aService,
+					 nsIDOMWindow* aParent,
+					 nsIChannel* aChannel,
+					 PRUint32 aLevel,
+					 nsIAuthInformation* aAuthInfo,
+					 const PRUnichar* aCheckLabel,
+					 PRBool* aCheckValue,
+					 PRBool* retval)
+{
+  // construct the message string
+  nsString message;
+  MakeDialogText(aChannel, aAuthInfo, message);
+
+  nsAutoString defaultUser, defaultDomain, defaultPass;
+  aAuthInfo->GetUsername(defaultUser);
+  aAuthInfo->GetDomain(defaultDomain);
+  aAuthInfo->GetPassword(defaultPass);
+
+  PRUint32 flags;
+  aAuthInfo->GetFlags(&flags);
+
+  if ((flags & nsIAuthInformation::NEED_DOMAIN) && !defaultDomain.IsEmpty()) {
+    defaultDomain.Append(PRUnichar('\\'));
+    defaultUser.Insert(defaultDomain, 0);
+  }
+
+  // NOTE: Allocation failure is not fatal here (just default to empty string
+  // if allocation fails)
+  PRUnichar *user = ToNewUnicode(defaultUser),
+    *pass = ToNewUnicode(defaultPass);
+  nsresult rv;
+  if (flags & nsIAuthInformation::ONLY_PASSWORD)
+    rv = aService->PromptPassword(aParent, nsnull, message.get(),
+                                  &pass, aCheckLabel,
+                                  aCheckValue, retval);
+  else
+    rv = aService->PromptUsernameAndPassword(aParent, nsnull, message.get(),
+                                             &user, &pass, aCheckLabel,
+                                             aCheckValue, retval);
+
+  nsString userStr(user);
+  nsString passStr(pass);
+  aAuthInfo->SetUsername(userStr);
+  aAuthInfo->SetPassword(passStr);
+
+  return rv;
+}
+
+/* boolean promptAuth (in nsIDOMWindow aParent, in nsIChannel aChannel, in PRUint32 level, in nsIAuthInformation authInfo, in wstring checkboxLabel, inout boolean checkValue); */
+NS_METHOD
+EphyPromptService::PromptAuth(nsIDOMWindow *aParent,
+			      nsIChannel *aChannel,
+			      PRUint32 level,
+			      nsIAuthInformation *authInfo,
+			      const PRUnichar *checkboxLabel,
+			      PRBool *checkValue,
+			      PRBool *_retval)
+{
+  NS_ENSURE_ARG_POINTER (_retval);
+  NS_ENSURE_ARG_POINTER (authInfo);
+
+  return EphyPromptService::PromptPasswordAdapter(this,
+						  aParent,
+						  aChannel,
+						  level,
+						  authInfo,
+						  checkboxLabel,
+						  checkValue,
+						  _retval);
+}
+
+/* nsICancelable asyncPromptAuth (in nsIDOMWindow aParent, in nsIChannel aChannel, in nsIAuthPromptCallback aCallback, in nsISupports aContext, in PRUint32 level, in nsIAuthInformation authInfo, in wstring checkboxLabel, inout boolean checkValue); */
+NS_METHOD EphyPromptService::AsyncPromptAuth(nsIDOMWindow *aParent,
+					     nsIChannel *aChannel,
+					     nsIAuthPromptCallback *aCallback,
+					     nsISupports *aContext,
+					     PRUint32 level,
+					     nsIAuthInformation *authInfo,
+					     const PRUnichar *checkboxLabel,
+					     PRBool *checkValue,
+					     nsICancelable **_retval)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+#endif /* HAVE_GECKO_1_9 */
