@@ -16,15 +16,13 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: GtkNSSDialogs.h 7960 2008-02-17 18:23:56Z chpe $
+ * $Id: GtkNSSDialogs.h 8323 2008-06-30 18:22:04Z chpe $
  */ 
 
 #ifndef GTKNSSDIALOGS_H
 #define GTKNSSDIALOGS_H 1
 
-#ifdef HAVE_GECKO_1_9
-#include <nsIBadCertListener2.h>
-#else
+#ifdef HAVE_NSIBADCERTLISTENER_H
 #include <nsIBadCertListener.h>
 #endif
 #include <nsICertificateDialogs.h>
@@ -42,7 +40,7 @@ class nsIPKCS11Slot;
 #define GTK_NSSDIALOGS_CLASSNAME  "Gtk NSS Dialogs"
 
 class GtkNSSDialogs : public nsICertificateDialogs,
-#ifndef HAVE_GECKO_1_9
+#ifdef HAVE_NSIBADCERTLISTENER_H
                       public nsIBadCertListener,
 #endif
 		      public nsITokenPasswordDialogs,
@@ -52,7 +50,7 @@ class GtkNSSDialogs : public nsICertificateDialogs,
   public:
     NS_DECL_ISUPPORTS
     NS_DECL_NSICERTIFICATEDIALOGS
-#ifndef HAVE_GECKO_1_9
+#ifdef HAVE_NSIBADCERTLISTENER_H
     NS_DECL_NSIBADCERTLISTENER
 #endif
     NS_DECL_NSITOKENPASSWORDDIALOGS
