@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-file-helpers.c 7888 2008-01-24 19:14:16Z cosimoc $
+ *  $Id: ephy-file-helpers.c 8030 2008-03-04 20:21:38Z wbolster $
  */
 
 #include "config.h"
@@ -713,7 +713,7 @@ ephy_file_browse_to (GFile *file,
 	desktop = g_file_new_for_path (desktop_dir);
 	
 	/* Don't do anything if destination is the desktop */
-	if (g_file_contains_file (desktop, file))
+	if (g_file_has_prefix (file, desktop))
 	{
 		ret = FALSE;
 	}
