@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-fullscreen-popup.c,v 1.11 2006/09/13 11:34:22 chpe Exp $
+ *  $Id: ephy-fullscreen-popup.c 7927 2008-02-10 18:26:22Z xan $
  */
 
 #include "config.h"
@@ -29,7 +29,6 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkeventbox.h>
-#include <gtk/gtktooltips.h>
 #include <gtk/gtkenums.h>
 #include <gtk/gtkbox.h>
 #include <gtk/gtkhbox.h>
@@ -43,7 +42,6 @@
 struct _EphyFullscreenPopupPrivate
 {
 	EphyWindow *window;
-	GtkTooltips *tooltips;
 	GtkWidget *frame;
 	EphySpinner *spinner;
 	GtkWidget *lock;
@@ -67,9 +65,9 @@ enum
 	LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL] = { 0 };
+static guint signals[LAST_SIGNAL];
 
-static GObjectClass *parent_class = NULL;
+G_DEFINE_TYPE (EphyFullscreenPopup, ephy_fullscreen_popup, GTK_TYPE_WINDOW)
 
 static void
 exit_button_clicked_cb (GtkWidget *button,
@@ -198,9 +196,9 @@ ephy_fullscreen_popup_set_security_state (EphyFullscreenPopup *popup,
 {
 	EphyFullscreenPopupPrivate *priv = popup->priv;
 
-	priv->show_lock = show_lock;
+	priv->show_lock = show_lock != FALSE;
 	gtk_image_set_from_stock (GTK_IMAGE (priv->lock), stock, GTK_ICON_SIZE_BUTTON);
-	gtk_tooltips_set_tip (priv->tooltips, priv->lock_ebox, tooltip, NULL);
+	gtk_widget_set_tooltip_text (priv->lock, tooltip);
 
 	ephy_fullscreen_popup_update_visibility (popup);
 }
@@ -229,8 +227,9 @@ ephy_fullscreen_popup_constructor (GType type,
 	GtkWindow *window;
 	GtkWidget *hbox, *frame_hbox, *icon;
 
-	object = parent_class->constructor (type, n_construct_properties,
-					    construct_params);
+	object = G_OBJECT_CLASS (ephy_fullscreen_popup_parent_class)->constructor (type,
+                                                                                   n_construct_properties,
+                                                                                   construct_params);
 
 	window = GTK_WINDOW (object);
 	popup = EPHY_FULLSCREEN_POPUP (window);
@@ -238,9 +237,6 @@ ephy_fullscreen_popup_constructor (GType type,
 
 	gtk_window_set_resizable (window, FALSE);
 
-	priv->tooltips = gtk_tooltips_new ();
-	g_object_ref_sink (priv->tooltips);
-
 	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_container_add (GTK_CONTAINER (window), hbox);
 	gtk_widget_show (hbox);
@@ -301,9 +297,7 @@ ephy_fullscreen_popup_finalize (GObject *object)
 	g_signal_handlers_disconnect_matched (priv->window, G_SIGNAL_MATCH_DATA,
 					      0, 0, NULL, NULL, popup);
 
-	g_object_unref (priv->tooltips);
-
-	parent_class->finalize (object);
+	G_OBJECT_CLASS (ephy_fullscreen_popup_parent_class)->finalize (object);
 }
 
 static void
@@ -337,7 +331,7 @@ ephy_fullscreen_popup_show (GtkWidget *widget)
 {
 	EphyFullscreenPopup *popup = EPHY_FULLSCREEN_POPUP (widget);
 
-	GTK_WIDGET_CLASS (parent_class)->show (widget);
+	GTK_WIDGET_CLASS (ephy_fullscreen_popup_parent_class)->show (widget);
 
 	ephy_fullscreen_popup_update_spinner (popup);
 }
@@ -347,7 +341,7 @@ ephy_fullscreen_popup_hide (GtkWidget *widget)
 {
 	EphyFullscreenPopup *popup = EPHY_FULLSCREEN_POPUP (widget);
 
-	GTK_WIDGET_CLASS (parent_class)->hide (widget);
+	GTK_WIDGET_CLASS (ephy_fullscreen_popup_parent_class)->hide (widget);
 
 	ephy_fullscreen_popup_update_spinner (popup);
 }
@@ -358,7 +352,7 @@ ephy_fullscreen_popup_size_request (GtkWidget *widget,
 {
 	EphyFullscreenPopup *popup = EPHY_FULLSCREEN_POPUP (widget);
 
-	GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+	GTK_WIDGET_CLASS (ephy_fullscreen_popup_parent_class)->size_request (widget, requisition);
 
 	if (GTK_WIDGET_REALIZED (widget))
 	{
@@ -371,7 +365,7 @@ ephy_fullscreen_popup_realize (GtkWidget *widget)
 {
 	EphyFullscreenPopup *popup = EPHY_FULLSCREEN_POPUP (widget);
 
-	GTK_WIDGET_CLASS (parent_class)->realize (widget);
+	GTK_WIDGET_CLASS (ephy_fullscreen_popup_parent_class)->realize (widget);
 
 	ephy_fullscreen_popup_update_position (popup);
 }
@@ -382,8 +376,6 @@ ephy_fullscreen_popup_class_init (EphyFullscreenPopupClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->constructor = ephy_fullscreen_popup_constructor;
 	object_class->finalize = ephy_fullscreen_popup_finalize;
 	object_class->get_property = ephy_fullscreen_popup_get_property;
@@ -427,34 +419,6 @@ ephy_fullscreen_popup_class_init (EphyFullscreenPopupClass *klass)
 	g_type_class_add_private (object_class, sizeof (EphyFullscreenPopupPrivate));
 }
 
-GType
-ephy_fullscreen_popup_get_type (void)
-{
-	static GType type = 0;
-
-	if (G_UNLIKELY (type == 0))
-	{
-		const GTypeInfo our_info =
-		{
-			sizeof (EphyFullscreenPopupClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) ephy_fullscreen_popup_class_init,
-			NULL,
-			NULL,
-			sizeof (EphyFullscreenPopup),
-			0,
-			(GInstanceInitFunc) ephy_fullscreen_popup_init
-		};
-
-		type = g_type_register_static (GTK_TYPE_WINDOW,
-					       "EphyFullscreenPopup",
-					       &our_info, 0);
-	}
-
-	return type;
-}
-
 GtkWidget *
 ephy_fullscreen_popup_new (EphyWindow *window)
 {
